@@ -1,53 +1,64 @@
 test_that("test file-backed raster caching", {
-  library(raster)
-
-  tmpdir <- file.path(tempdir(), "test_Cache") %>% checkPath(create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
-  tmpRasterfile <- tempfile(tmpdir = tmpdir, fileext = ".tif")
-  file.create(tmpRasterfile)
-  tmpRasterfile <- normPath(tmpRasterfile)
-  try(clearCache(tmpdir), silent = TRUE)
+  testInitOut <- testInit("raster", tmpFileExt = c(".tif", ".grd"))
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
 
   nOT <- Sys.time()
 
-  randomPolyToDisk <- function(tmpRasterfile) {
+  randomPolyToDisk <- function(tmpfile) {
     r <- raster(extent(0, 10, 0, 10), vals = sample(1:30, size = 100, replace = TRUE))
-    writeRaster(r, tmpRasterfile, overwrite = TRUE)
-    r <- raster(tmpRasterfile)
+    writeRaster(r, tmpfile[1], overwrite = TRUE)
+    r <- raster(tmpfile[1])
     r
   }
 
-  a <- randomPolyToDisk(tmpRasterfile)
+  a <- randomPolyToDisk(tmpfile[1])
   # confirm that the raster has the given tmp filename
-  expect_identical(strsplit(tmpRasterfile, split = "[\\/]"),
+  expect_identical(strsplit(tmpfile[1], split = "[\\/]"),
                    strsplit(a@file@name, split = "[\\/]"))
-  aa <- Cache(randomPolyToDisk, tmpRasterfile, cacheRepo = tmpdir, userTags = "something2")
+  aa <- Cache(randomPolyToDisk, tmpfile[1], cacheRepo = tmpdir, userTags = "something2")
 
   # Test clearCache by tags
   expect_equal(NROW(showCache(tmpdir)[tagKey != "otherFunctions"]), 11)
-  clearCache(tmpdir, userTags = "something$")
+  clearCache(tmpdir, userTags = "something$", ask = FALSE)
   expect_equal(NROW(showCache(tmpdir)[tagKey != "otherFunctions"]), 11)
-  clearCache(tmpdir, userTags = "something2")
+  clearCache(tmpdir, userTags = "something2", ask = FALSE)
   expect_equal(NROW(showCache(tmpdir)), 0)
 
-  aa <- Cache(randomPolyToDisk, tmpRasterfile, cacheRepo = tmpdir, userTags = "something2")
+  aa <- Cache(randomPolyToDisk, tmpfile[1], cacheRepo = tmpdir, userTags = "something2")
   expect_equal(NROW(showCache(tmpdir)[tagKey != "otherFunctions"]), 11)
-  clearCache(tmpdir, userTags = c("something$", "testing$"))
+  clearCache(tmpdir, userTags = c("something$", "testing$"), ask = FALSE)
   expect_equal(NROW(showCache(tmpdir)[tagKey != "otherFunctions"]), 11)
-  clearCache(tmpdir, userTags = c("something2$", "testing$"))
+  clearCache(tmpdir, userTags = c("something2$", "testing$"), ask = FALSE)
   expect_equal(NROW(showCache(tmpdir)[tagKey != "otherFunctions"]), 11)
-  clearCache(tmpdir, userTags = c("something2$", "randomPolyToDisk$"))
+  clearCache(tmpdir, userTags = c("something2$", "randomPolyToDisk$"), ask = FALSE)
   expect_equal(NROW(showCache(tmpdir)), 0)
 
-  aa <- Cache(randomPolyToDisk, tmpRasterfile, cacheRepo = tmpdir, userTags = "something2")
+  aa <- Cache(randomPolyToDisk, tmpfile[1], cacheRepo = tmpdir, userTags = "something2")
 
   # confirm that the raster has the new filename in the cachePath
-  expect_false(identical(strsplit(tmpRasterfile, split = "[\\/]"),
+  expect_false(identical(strsplit(tmpfile[1], split = "[\\/]"),
                          strsplit(file.path(tmpdir, "rasters",
-                                            basename(tmpRasterfile)), split = "[\\/]")))
-  expect_true(any(grepl(pattern = basename(tmpRasterfile),
+                                            basename(tmpfile[1])), split = "[\\/]")))
+  expect_true(any(grepl(pattern = basename(tmpfile[1]),
                         dir(file.path(tmpdir, "rasters")))))
 
+  ### Test for 2 caching events with same file-backing name
+  randomPolyToDisk2 <- function(tmpfile, rand) {
+    r <- raster(extent(0, 10, 0, 10), vals = sample(1:30, size = 100, replace = TRUE))
+    writeRaster(r, tmpfile[1], overwrite = TRUE)
+    r <- raster(tmpfile[1])
+    r
+  }
+
+  a <- Cache(randomPolyToDisk2, tmpfile[1], runif(1))
+  b <- Cache(randomPolyToDisk2, tmpfile[1], runif(1))
+
+  expect_false(identical(filename(a), filename(b)))
+
+
+
   # Caching a raster as an input works
   rasterTobinary <- function(raster) {
     ceiling(raster[] / (mean(raster[]) + 1))
@@ -60,15 +71,15 @@ test_that("test file-backed raster caching", {
     nOT <- Sys.time() - 100
   }
 
-  attr(a1, "newCache") <- NULL
-  attr(a2, "newCache") <- NULL
+  attr(a1, ".Cache")$newCache <- NULL
+  attr(a2, ".Cache")$newCache <- NULL
   # test that they are identical
   expect_equal(a1, a2)
 
   # confirm that the second one was obtained through reading from Cache... much faster than writing
   expect_true(b1[1] > b2[1])
 
-  clearCache(tmpdir)
+  clearCache(tmpdir, ask = FALSE)
 
   # Check that Caching of rasters saves them to tif file instead of rdata
   randomPolyToMemory <- function() {
@@ -99,42 +110,39 @@ test_that("test file-backed raster caching", {
   expect_true(NCOL(raster::levels(bb)[[1]]) == 3)
   expect_true(NROW(raster::levels(bb)[[1]]) == 30)
 
-  randomPolyToFactorOnDisk <- function(tmpFile) {
+  randomPolyToFactorOnDisk <- function(tmpfile) {
     r <- raster(extent(0, 10, 0, 10), vals = sample(1:30, size = 100, replace = TRUE))
     levels(r) <- data.frame(ID = 1:30, vals = sample(LETTERS[1:5], size = 30, replace = TRUE),
                             vals2 = sample(1:7, size = 30, replace = TRUE))
-    r <- writeRaster(r, tmpFile, overwrite = TRUE, datatype = "INT1U")
+    r <- writeRaster(r, tmpfile, overwrite = TRUE, datatype = "INT1U")
     r
   }
-  tf <- tempfile(fileext = ".grd")
-  file.create(tf)
-  tf <- normPath(tf)
 
   # bb1 has original tmp filename
-  bb1 <- randomPolyToFactorOnDisk(tf)
+  bb1 <- randomPolyToFactorOnDisk(tmpfile[2])
   # bb has new one, inside of cache repository, with same basename
-  bb <- Cache(randomPolyToFactorOnDisk, tmpFile = tf, cacheRepo = tmpdir)
-  expect_true(dirname(filename(bb)) == file.path(tmpdir, "rasters"))
-  expect_true(basename(filename(bb)) == basename(tf))
-  expect_false(filename(bb) == tf)
-  expect_true(dirname(filename(bb1)) == dirname(tf))
-  expect_true(basename(filename(bb1)) == basename(tf))
+  bb <- Cache(randomPolyToFactorOnDisk, tmpfile = tmpfile[2], cacheRepo = tmpdir)
+  expect_true(dirname(normPath(filename(bb))) == normPath(file.path(tmpdir, "rasters")))
+  expect_true(basename(filename(bb)) == basename(tmpfile[2]))
+  expect_false(filename(bb) == tmpfile[2])
+  expect_true(dirname(filename(bb1)) == dirname(tmpfile[2]))
+  expect_true(basename(filename(bb1)) == basename(tmpfile[2]))
   expect_true(dataType(bb) == "INT1U")
   expect_true(raster::is.factor(bb))
   expect_true(is(raster::levels(bb)[[1]], "data.frame"))
   expect_true(NCOL(raster::levels(bb)[[1]]) == 3)
   expect_true(NROW(raster::levels(bb)[[1]]) == 30)
 
-  clearCache(tmpdir)
+  clearCache(tmpdir, ask = FALSE)
 })
 
 test_that("test memory backed raster robustDigest", {
-  library(raster)
-  tmpdir <- file.path(tempdir(), "testCache") %>% checkPath(create = TRUE)
-  tmpFile <- tempfile()
-  tmpFile1 <- tempfile()
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
-
+  testInitOut <- testInit("raster", tmpFileExt = c("",""))
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
+  # library(raster)
+  # tmpdir <- file.path(tempdir(), "testCache") %>% checkPath(create = TRUE)
   set.seed(123)
   r1 <- raster(extent(0, 10, 0, 10), vals = sample(1:30, size = 100, replace = TRUE))
   r2 <- raster(extent(0, 10, 0, 10), vals = sample(1:30, size = 100, replace = TRUE))
@@ -164,12 +172,12 @@ test_that("test memory backed raster robustDigest", {
 
   expect_identical(dig, dig1)
 
-  b <- writeRaster(b, file = tmpFile, overwrite = TRUE)
+  b <- writeRaster(b, file = tmpfile[1], overwrite = TRUE)
   dig <- reproducible:::.robustDigest(b)
 
   r <- raster(matrix(1:10, 2, 5))
   b <- brick(r, r)
-  b <- writeRaster(b, file = tmpFile, overwrite = TRUE)
+  b <- writeRaster(b, file = tmpfile[1], overwrite = TRUE)
   dig1 <- reproducible:::.robustDigest(b)
 
   expect_identical(dig, dig1)
@@ -186,33 +194,30 @@ test_that("test memory backed raster robustDigest", {
 
   expect_identical(dig, dig1)
 
-  r4 <- writeRaster(r, file = tmpFile, overwrite = TRUE)
-  r5 <- writeRaster(r, file = tmpFile1, overwrite = TRUE)
+  r4 <- writeRaster(r, file = tmpfile[1], overwrite = TRUE)
+  r5 <- writeRaster(r, file = tmpfile[2], overwrite = TRUE)
   b <- raster::stack(r4, r5)
   dig <- reproducible:::.robustDigest(b)
 
-  r2 <- writeRaster(r1, file = tmpFile, overwrite = TRUE)
-  r3 <- writeRaster(r1, file = tmpFile1, overwrite = TRUE)
+  r2 <- writeRaster(r1, file = tmpfile[1], overwrite = TRUE)
+  r3 <- writeRaster(r1, file = tmpfile[2], overwrite = TRUE)
   b1 <- raster::stack(r2, r3)
-  #b1 <- writeRaster(b1, file = tmpFile, overwrite = TRUE)
+  #b1 <- writeRaster(b1, file = tmpfile[1], overwrite = TRUE)
   dig1 <- reproducible:::.robustDigest(b1)
 
   expect_identical(dig, dig1)
 })
 
 test_that("test 'quick' argument", {
-  library(raster)
-  tmpdir <- file.path(tempdir(), "testCache") %>% checkPath(create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
+  testInitOut <- testInit("raster", tmpFileExt = ".tif")
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
 
   ### Make raster using Cache
   set.seed(123)
   r1 <- raster(extent(0, 10, 0, 10), vals = sample(1:30, size = 100, replace = TRUE))
-  #r2 <- raster(extent(0, 10, 0, 10), vals = sample(1:30, size = 100, replace = TRUE))
-  tmpFile1 <- tempfile(fileext = ".tif")
-  #tmpFile2 <- tempfile()
-  r1 <- writeRaster(r1, filename = tmpFile1, overwrite = TRUE)
-  #r2 <- writeRaster(r2, filename = tmpFile2, overwrite = TRUE)
+  r1 <- writeRaster(r1, filename = tmpfile, overwrite = TRUE)
   quickFun <- function(rasFile) {
     ras <- raster(rasFile)
     ras[sample(ncell(ras), size = 1)]
@@ -221,7 +226,7 @@ test_that("test 'quick' argument", {
   out1a <- Cache(quickFun, asPath(filename(r1)), cacheRepo = tmpdir)
   out1b <- Cache(quickFun, asPath(filename(r1)), cacheRepo = tmpdir, quick = TRUE)
   r1[4] <- r1[4] + 1
-  r1 <- writeRaster(r1, filename = tmpFile1, overwrite = TRUE)
+  r1 <- writeRaster(r1, filename = tmpfile, overwrite = TRUE)
   mess1 <- capture_message(out1c <- Cache(quickFun, asPath(filename(r1)),
                                           cacheRepo = tmpdir, quick = TRUE))
   expect_true(grepl("loading cached result from previous quickFun call, adding to memoised copy", mess1 ))
@@ -236,7 +241,7 @@ test_that("test 'quick' argument", {
   out1a <- Cache(quickFun, r1, cacheRepo = tmpdir)
   out1b <- Cache(quickFun, r1, cacheRepo = tmpdir, quick = TRUE)
   r1[4] <- r1[4] + 1
-  r1 <- writeRaster(r1, filename = tmpFile1, overwrite = TRUE)
+  r1 <- writeRaster(r1, filename = tmpfile, overwrite = TRUE)
   mess1 <- capture_message(out1c <- Cache(quickFun, r1, cacheRepo = tmpdir, quick = TRUE))
   expect_true(grepl("loading cached result from previous quickFun call, adding to memoised copy", mess1 ))
   mess1 <- capture_message(out1c <- Cache(quickFun, r1, cacheRepo = tmpdir, quick = FALSE))
@@ -246,50 +251,44 @@ test_that("test 'quick' argument", {
 })
 
 test_that("test date-based cache removal", {
-  tmpdir <- file.path(tempdir(), "testCache") %>% checkPath(create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
-
-  tmpfile <- tempfile(fileext = ".pdf")
-  file.create(tmpfile)
-  tmpfile <- normPath(tmpfile)
-  try(clearCache(tmpdir), silent = TRUE)
+  testInitOut <- testInit("raster", tmpFileExt = ".pdf")
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
 
   a <- Cache(runif, 1, cacheRepo = tmpdir)
   a1 <- showCache(tmpdir)
   expect_true(NROW(a1) > 0)
-  b <- clearCache(tmpdir, before = Sys.Date() - 1)
+  b <- clearCache(tmpdir, before = Sys.Date() - 1, ask = FALSE)
   expect_true(NROW(b) == 0)
   expect_identical(a1, showCache(tmpdir))
 
-  b <- clearCache(tmpdir, before = Sys.Date() + 1)
-  expect_identical(b, a1)
+  b <- clearCache(tmpdir, before = Sys.Date() + 1, ask = FALSE)
+  expect_identical(data.table::setindex(b, NULL), a1)
 })
 
 test_that("test keepCache", {
-  library(raster)
-
-  tmpdir <- file.path(tempdir(), "testCache") %>% checkPath(create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
-
-  try(clearCache(tmpdir), silent = TRUE)
+  testInitOut <- testInit("raster")
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
   Cache(rnorm, 10, cacheRepo = tmpdir)
   Cache(runif, 10, cacheRepo = tmpdir)
   Cache(round, runif(4), cacheRepo = tmpdir)
   expect_true(NROW(showCache(tmpdir)[tagKey != "otherFunctions"]) == 30)
   expect_true(NROW(showCache(tmpdir, c("rnorm", "runif"))) == 0) # and search
-  expect_true(NROW(keepCache(tmpdir, "rnorm")[tagKey != "otherFunctions"]) == 10)
+  expect_true(NROW(keepCache(tmpdir, "rnorm", ask = FALSE)[tagKey != "otherFunctions"]) == 10)
 
   # do it twice to make sure it can deal with repeats
-  expect_true(NROW(keepCache(tmpdir, "rnorm")[tagKey != "otherFunctions"]) == 10)
+  expect_true(NROW(keepCache(tmpdir, "rnorm", ask = FALSE)[tagKey != "otherFunctions"]) == 10)
   Sys.sleep(1)
   st <- Sys.time()
   Sys.sleep(1)
   Cache(sample, 10, cacheRepo = tmpdir)
   Cache(length, 10, cacheRepo = tmpdir)
   Cache(sum, runif(4), cacheRepo = tmpdir)
-  showCache(tmpdir, after = st)
   expect_true(NROW(showCache(tmpdir, before = st)[tagKey != "otherFunctions"]) == 10)
-  expect_true(NROW(keepCache(tmpdir, before = st)[tagKey != "otherFunctions"]) == 10)
+  expect_true(NROW(keepCache(tmpdir, before = st, ask = FALSE)[tagKey != "otherFunctions"]) == 10)
   expect_true(NROW(showCache(tmpdir)[tagKey != "otherFunctions"]) == 10)
 
   ranNums <- Cache(runif, 4, cacheRepo = tmpdir, userTags = "objectName:a")
@@ -303,19 +302,19 @@ test_that("test keepCache", {
   expect_true(length(unique(showCache(tmpdir, userTags = "spades|rnorm")$artifact)) == 2)
 
   # keep all with spades or rnorm
-  expect_true(length(unique(keepCache(tmpdir, userTags = "spades|rnorm")$artifact)) == 2)
+  expect_true(length(unique(
+    keepCache(tmpdir, userTags = "spades|rnorm", ask = FALSE)$artifact)) == 2)
 
   # shows spades, runif and rnorm objects
   expect_true(length(unique(showCache(tmpdir)$artifact)) == 2)
 })
 
 test_that("test environments", {
-  library(raster)
+  testInitOut <- testInit("raster", tmpFileExt = ".pdf")
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
 
-  tmpdir <- file.path(tempdir(), "testCache") %>% checkPath(create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
-
-  try(clearCache(tmpdir), silent = TRUE)
   # make several unique environments
   a <- new.env()
   b <- new.env()
@@ -337,7 +336,7 @@ test_that("test environments", {
   out <- Cache(shortFn, a = a, cacheRepo = tmpdir)
   out2 <- Cache(shortFn, a = b, cacheRepo = tmpdir)
   out3 <- Cache(shortFn, a = g, cacheRepo = tmpdir)
-  attr(out2, "newCache") <- TRUE
+  attr(out2, ".Cache")$newCache <- TRUE
   expect_true(identical(attributes(out)["tags"], attributes(out2)["tags"]))
   expect_true(identical(attributes(out)["tags"], attributes(out3)["tags"]))
 
@@ -375,12 +374,11 @@ test_that("test environments", {
 })
 
 test_that("test asPath", {
-  library(raster)
+  testInitOut <- testInit("raster", tmpFileExt = "pdf")
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
 
-  tmpdir <- file.path(tempdir(), "testCache") %>% checkPath(create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
-
-  try(clearCache(tmpdir), silent = TRUE)
   obj <- 1:10
   origDir <- getwd()
   on.exit(setwd(origDir))
@@ -398,7 +396,7 @@ test_that("test asPath", {
   expect_true(grepl("loading cached", a3))
 
   unlink("filename.RData")
-  try(clearCache(tmpdir), silent = TRUE)
+  try(clearCache(tmpdir, ask = FALSE), silent = TRUE)
   a1 <- capture_messages(Cache(saveRDS, obj, file = asPath("filename.RData"),
                                quick = TRUE, cacheRepo = tmpdir))
   a2 <- capture_messages(Cache(saveRDS, obj, file = asPath("filename.RData"),
@@ -410,7 +408,7 @@ test_that("test asPath", {
   expect_true(grepl("loading memoised result from previous saveRDS call", a3))
 
   unlink("filename.RData")
-  try(clearCache(tmpdir), silent = TRUE)
+  try(clearCache(tmpdir, ask = FALSE), silent = TRUE)
   a1 <- capture_messages(Cache(saveRDS, obj, file = as("filename.RData", "Path"),
                                quick = TRUE, cacheRepo = tmpdir))
   a2 <- capture_messages(Cache(saveRDS, obj, file = as("filename.RData", "Path"),
@@ -421,16 +419,17 @@ test_that("test asPath", {
   expect_true(grepl("loading cached", a2))
   expect_true(grepl("loading memoised result from previous saveRDS call", a3))
 
-  setwd(origDir)
-  unlink(tmpdir, recursive = TRUE)
+  # setwd(origDir)
+  # unlink(tmpdir, recursive = TRUE)
 
   # make several unique environments
 })
 
 test_that("test wrong ways of calling Cache", {
-  tmpdir <- file.path(tempdir(), "testCache")
-  checkPath(tmpdir, create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
+  testInitOut <- try(testInit(tmpFileExt = ".pdf"))
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
 
   expect_error(Cache(sample(1), cacheRepo = tmpdir), "Can't understand")
   expect_error(Cache(a <- sample(1), cacheRepo = tmpdir), "Can't understand")
@@ -438,10 +437,13 @@ test_that("test wrong ways of calling Cache", {
 })
 
 test_that("test pipe for Cache", {
-  tmpdir <- file.path(tempdir(), "testCache")
-  checkPath(tmpdir, create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
-
+  skip_on_cran()
+  skip_on_travis()
+  skip("Not possible to test automatically ... testthat now exports pipe")
+  testInitOut <- testInit("raster", tmpFileExt = ".pdf")
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
   a <- rnorm(10, 16) %>% mean() %>% prod(., 6) # nolint
   b <- rnorm(10, 16) %>% mean() %>% prod(., 6) %>% Cache(cacheRepo = tmpdir) # nolint
   d <- rnorm(10, 16) %>% mean() %>% prod(., 6) %>% Cache(cacheRepo = tmpdir) # nolint
@@ -461,7 +463,7 @@ test_that("test pipe for Cache", {
 
 
   # New Pipe
-  clearCache(tmpdir)
+  clearCache(tmpdir, ask = FALSE)
   a <- rnorm(10, 16) %>% mean() %>% prod(., 6) # nolint
   b <- Cache(cacheRepo = tmpdir) %C% rnorm(10, 16) %>% mean() %>% prod(., 6) # nolint
   d <- Cache(cacheRepo = tmpdir) %C% rnorm(10, 16) %>% mean() %>% prod(., 6) # nolint
@@ -481,7 +483,7 @@ test_that("test pipe for Cache", {
   expect_false(isTRUE(all.equalWONewCache(d1, d2)))
 
 
-  clearCache(tmpdir)
+  clearCache(tmpdir, ask = FALSE)
   a <- rnorm(10, 16) %>% mean() %>% prod(., 6) # nolint
   b <- Cache(cacheRepo = tmpdir) %C% rnorm(10, 16) %>% mean() # nolint
   d <- Cache(cacheRepo = tmpdir) %C% rnorm(10, 16) %>% mean() # nolint
@@ -490,26 +492,31 @@ test_that("test pipe for Cache", {
 })
 
 test_that("test quoted FUN in Cache", {
-  tmpdir <- file.path(tempdir(), "testCache")
-  checkPath(tmpdir, create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
+  testInitOut <- testInit()
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
 
   A <- Cache(rnorm, 10, 16, cacheRepo = tmpdir) # nolint
 
   ## recover cached copies:
   B <- Cache(rnorm, 10, 16, cacheRepo = tmpdir) # nolint
   C <- Cache(quote(rnorm(n = 10, 16)), cacheRepo = tmpdir) # nolint
-  D <- rnorm(10, 16) %>% Cache(cacheRepo = tmpdir) # nolint
+
+  D <- try(rnorm(10, 16) %>% Cache(cacheRepo = tmpdir), silent = TRUE) # nolint
 
   expect_true(all.equalWONewCache(A,B))
   expect_true(all.equalWONewCache(A, C))
-  expect_true(all.equalWONewCache(A, D))
+  if (!is(D, "try-error"))
+    expect_true(all.equalWONewCache(A, D))
 })
 
 test_that("test multiple pipe Cache calls", {
-  tmpdir <- file.path(tempdir(), "testCache")
-  checkPath(tmpdir, create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
+  skip("Impossible to test automatically, testthat now exports %>%")
+  testInitOut <- testInit()
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
 
   d <- list()
   mess <- list()
@@ -567,10 +574,10 @@ test_that("test multiple pipe Cache calls", {
 
 
 test_that("test masking of %>% error message", {
-  tmpdir <- file.path(tempdir(), "testCache")
-  checkPath(tmpdir, create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
-  on.exit(try(detach("package:magrittr"), silent = TRUE), add = TRUE)
+  testInitOut <- testInit()
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
 
   mess <- capture_messages(library(magrittr))
 
@@ -601,16 +608,17 @@ test_that("test masking of %>% error message", {
 
 
 test_that("test Cache argument inheritance to inner functions", {
-  cacheDir <- paste(sample(letters, 5), collapse = "")
-  tmpdir <- file.path(tempdir(), cacheDir)
-  checkPath(tmpdir, create = TRUE)
-  on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)
+  testInitOut <- testInit("raster")
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
+
 
   outer <- function(n) {
     Cache(rnorm, n)
   }
   expect_silent(Cache(outer, n = 2, cacheRepo = tmpdir))
-  clearCache(tmpdir)
+  clearCache(tmpdir, ask = FALSE)
 
   out <- capture_messages(Cache(outer, n = 2))
   expect_true(all(unlist(lapply(c("No cacheRepo supplied. Using value in getOption\\('reproducible.cachePath'\\)",
@@ -690,7 +698,7 @@ test_that("test Cache argument inheritance to inner functions", {
   expect_true(all(grepl("loading cached result from previous rnorm call", out)))
 
   # Check userTags -- all items have it
-  clearCache(tmpdir)
+  clearCache(tmpdir, ask = FALSE)
   outerTag <- "howdie"
   aa <- Cache(outer, n = 2, cacheRepo = tmpdir, userTags = outerTag)
   bb <- showCache(tmpdir, userTags = outerTag)
@@ -704,7 +712,7 @@ test_that("test Cache argument inheritance to inner functions", {
     Cache(rnorm, n = 3, mean = mean, notOlderThan = Sys.time() - 1e5, userTags = innerTag)
   }
 
-  clearCache(tmpdir)
+  clearCache(tmpdir, ask = FALSE)
   aa <- Cache(outer, n = 2, cacheRepo = tmpdir, userTags = outerTag)
   bb <- showCache(tmpdir, userTags = outerTag)
   cc <- showCache(tmpdir)
@@ -724,14 +732,11 @@ test_that("test Cache argument inheritance to inner functions", {
 
 ##########################
 test_that("test reproducible.verbose", {
-  cacheDir <- paste(sample(letters, 5), collapse = "")
-  tmpdir <- file.path(tempdir(), cacheDir)
-  checkPath(tmpdir, create = TRUE)
-  options(reproducible.verbose = TRUE)
+  testInitOut <- testInit("raster", verbose = TRUE, tmpFileExt = ".rds")
   on.exit({
-    options(reproducible.verbose = FALSE)
-    unlink(tmpdir, recursive = TRUE)
-    }, add = TRUE)
+    testOnExit(testInitOut)
+  }, add = TRUE)
+
   Cache(rnorm, 1, cacheRepo = tmpdir)
   expect_is(.reproEnv$cacheTimings, "data.frame")
   expect_true(NROW(.reproEnv$cacheTimings)==4)
@@ -740,11 +745,10 @@ test_that("test reproducible.verbose", {
   # Test Path class objects
   a <- sample(1e4)
 
-  tmpFile <- tempfile(fileext = ".rds")
-  saveRDS(a, file = tmpFile)
-  out1 <- Cache(readRDS, tmpFile, cacheRepo = tmpdir)
+  saveRDS(a, file = tmpfile)
+  out1 <- Cache(readRDS, tmpfile, cacheRepo = tmpdir)
   out1Details <- .reproEnv$hashDetailsAll
-  out2 <- Cache(readRDS, asPath(tmpFile), cacheRepo = tmpdir)
+  out2 <- Cache(readRDS, asPath(tmpfile), cacheRepo = tmpdir)
   out2Details <- .reproEnv$hashDetailsAll
 
   # should be vastly larger when actual file, rather than just filename
@@ -756,44 +760,30 @@ test_that("test reproducible.verbose", {
 
 
 ##########################
-test_that("test reproducible.verbose", {
-  cacheDir1 <- paste(sample(letters, 5), collapse = "")
-  cacheDir2 <- paste(sample(letters, 5), collapse = "")
+test_that("test future", {
+  skip_on_cran()
+  .onLinux <- .Platform$OS.type == "unix" && unname(Sys.info()["sysname"]) == "Linux"
+  if (.onLinux) {
+    if (requireNamespace("future", quietly = TRUE)) {
+      testInitOut <- testInit("raster", verbose = TRUE, tmpFileExt = ".rds")
+      on.exit({
+        testOnExit(testInitOut)
+      }, add = TRUE)
 
-  cacheDir1 <- file.path(tempdir(), cacheDir1)
-  cacheDir2 <- file.path(tempdir(), cacheDir2)
+      options("reproducible.futurePlan" = "multiprocess")
+      (aa <- system.time({for(i in c(1:3)) a <- Cache(cacheRepo = tmpCache, seq, 5, 1e7 + i)}))
+      a <- showCache(tmpCache)
 
-  checkPath(cacheDir1, create = TRUE)
-  checkPath(cacheDir2, create = TRUE)
+      options("reproducible.futurePlan" = FALSE)
+      try(unlink(tmpCache, recursive = TRUE))
+      (bb <- system.time({for(i in 1:3) a <- Cache(cacheRepo = tmpCache, seq, 5, 1e7 + i)}))
 
-  options(reproducible.verbose = FALSE)
+      # Test the speed of rerunning same line
+      (aa <- system.time({for(i in c(1,1)) a <- Cache(cacheRepo = tmpCache, seq, 5, 1e7 + i)}))
 
-  on.exit({
-    options(reproducible.verbose = FALSE)
-    unlink(cacheDir1, recursive = TRUE)
-    unlink(cacheDir2, recursive = TRUE)
-  }, add = TRUE)
-  Cache(rnorm, 1, cacheRepo = cacheDir1)
-  Cache(rnorm, 2, cacheRepo = cacheDir2)
-  Cache(rnorm, 3, cacheRepo = cacheDir1)
-  Cache(rnorm, 4, cacheRepo = cacheDir2)
-  library(raster)
-  r <- raster(extent(0,10,0,10), res = 1, vals = 1:100)
-  r <- Cache(writeRaster, r, filename = tempfile(fileext = ".tif"), cacheRepo = cacheDir2)
-  preCacheDir1Items <- data.table::copy(showCache(cacheDir1))
-  preCacheDir2Items <- data.table::copy(showCache(cacheDir2))
-  cacheDir1 <- mergeCache(cacheDir1, cacheDir2)
-  postCache <- showCache(cacheDir1)[tagKey != "date"]
-  expect_true(identical(sort(unique(postCache[tagKey=="cacheId"]$tagValue)),
-                      sort(c(unique(preCacheDir1Items[tagKey=="cacheId"]$tagValue),
-                        unique(preCacheDir2Items[tagKey=="cacheId"]$tagValue)))))
-  preCacheRbindlist <- data.table::setkey(data.table::rbindlist(
-    list(preCacheDir1Items[tagKey != "date"], preCacheDir2Items[tagKey != "date"])), artifact)
-
-  postCache[,`:=`(artifact=NULL, createdDate=NULL)]
-  data.table::setkey(postCache, tagKey, tagValue)
-  preCacheRbindlist[,`:=`(artifact=NULL, createdDate=NULL)]
-  data.table::setkey(preCacheRbindlist, tagKey, tagValue)
-  expect_true(all.equal(postCache,preCacheRbindlist))
+    }
 
+  }
 })
+
+
